/* 
 * \file        ComplexFiltersNilssonBigun.h
 * \author      Mikel Galar Idoate (mikel.galar@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * Header file for ComplexFiltersNilssonBigun  class
 *
 * 
 */

#ifndef COMPLEXFILTERSNILSSONBIGUN_H
#define	COMPLEXFILTERSNILSSONBIGUN_H


#include "PCASYSManager.h"

class ComplexFiltersNilssonBigun {
public:
    ComplexFiltersNilssonBigun();
    
    ~ComplexFiltersNilssonBigun();
    
    void initialize(float **img, int h, int w);
    
    /**
     * Executes the Complex Filters Nilsson and Bigun method from the fingerprint in file
     * \param file fingerprint
     * \param segmentar indicates if Poincare's segmentation must be done
     */
    void run(string file, PCASYSManager *pcasys, bool segmentationPoincare = false);
    
    void constructGaussianPyramid(float sigma);

    void computeOrientationMap(int blockW);
    
    void computeFilterResponses(float sigma, float sigmag1, float sigmag2Percentage);

    void computeCoresAndDeltas(float thresholdCores, float thresholdDeltas);
    
    vector<int> findLocalMaximum(float **matrix, int h, int w, float threshold);
    
    vector<int> findInLowerLevel(vector<int> localMaximum, float **matrix, int h, int w, float threshold, int s, float *magnitud, bool core = false);

    
    vector<int> computePixelPosition(vector<int> singularPoints);
    
    vector<float> computeAngles(vector<int> singularPoints, int type);
    
        /**
     * Gets the centroids of the cores of the image. Each even position is a vertical component, while
     *      each odd one is a horizontal one. I.e. the coordinates of the first core are
     *      the positions 0 (vertical) and 1 (horizontal), the coordinates of the second one
     *      are in the positions 2 and 3, and so on.
     * The coordinates are given IN TERMS OF PIXELS, NOT BLOCKS
     * \return the vector containing all the sizes
     */
    vector<int> getCores();

    /**
     * Gets the centroids of the deltas of the image. Each even position is a vertical component, while
     *      each odd one is a horizontal one. I.e. the coordinates of the first delta are
     *      the positions 0 (vertical) and 1 (horizontal), the coordinates of the second one
     *      are in the positions 2 and 3, and so on.
     * The coordinates are given IN TERMS OF PIXELS, NOT BLOCKS
     * \return the vector containing all the sizes
     */
    vector<int> getDeltas();
    
    vector<float> getCoreAngles();
    
    vector<float> getDeltaAngles();
    
    float getDeltaMaxMag();
    
    float getCoreMaxMag();
   

private:
    
    float **img;
    int h;
    int w;
    
    float ****filterResponseX;
    float ****filterResponseY;
    
    float ****filteredMagnitude;
    float ****filteredMagnitudeC;
    float ****filteredAngle;
    
    float ***O;
    float ***xCos;
    float ***ySin;
    int *orH;
    int *orW;
    
    bool **segmentation;
    
    int nLevels;
    
    float coreMaxMag;
    float deltaMaxMag;
    
    vector<int> cores;
    vector<int> deltas;
    vector <float> coreAngles;
    vector <float> deltaAngles;
    

};


inline vector<int> ComplexFiltersNilssonBigun::getCores() { return cores; }
inline vector<int> ComplexFiltersNilssonBigun::getDeltas() { return deltas; }
inline vector<float> ComplexFiltersNilssonBigun::getCoreAngles() { return coreAngles; }
inline vector<float> ComplexFiltersNilssonBigun::getDeltaAngles() { return deltaAngles; }
inline float ComplexFiltersNilssonBigun::getDeltaMaxMag() { return deltaMaxMag;}
inline float ComplexFiltersNilssonBigun::getCoreMaxMag() { return coreMaxMag;}

#endif	/* COMPLEXFILTERSNILSSONBIGUN_H */

