/* 
 * \file        ComputeLabels.cpp
 * \author      Juan Cerrón González (juan.cerron@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * The implementation of ComputeLabels class
 *
 * 
 */

#include "ComputeLabels.h"
#include "MemUtil.hpp"
#include "ImgUtil.h"
#include "NewPoincareSingularPoints.h"
#include "ParametersZhang.h"



ComputeLabels::ComputeLabels() {
    
    img = 0;
    labelImg = 0;
    h = 0;
    w = 0;
    nComponents = 0;
    lastLabel = 0;
    
}

ComputeLabels::~ComputeLabels() {
    
    MemUtil::freeMatrix(&img);
    MemUtil::freeMatrix(&labelImg);
    MemUtil::freeMatrix(&nComponents);
}

int ComputeLabels::calculateLabels(int **imagen, int hI, int wI) {

    this->h = hI;
    this->w = wI;
    
    img = MemUtil::copy2DMatrix<int>(imagen,h,w);
    
    labelImg = MemUtil::create2DMatrix<int>(h, w);
    nComponents = MemUtil::createVector<int>(maxLabelsZhang);
    
    for (int i = 0; i < h; i++) {
        for (int j = 0; j < w; j++) {
            
            //Si el pixel de la imagen no es fondo y el label de ese pixel está
            //sin asignar..
            if ((img[i][j] == 1) && (labelImg[i][j] == 0)) {
                if (lastLabel==maxLabelsZhang){
                    //cout << "Demasiados componentes encontrados: " << lastLabel << endl;
                    return -1;
                }
                lastLabel++;
                labelImg[i][j] = lastLabel;
                
                //Aumentamos el nº de componentes del label
                nComponents[lastLabel]++;
                fillLabelRegion(i, j);                                

            }
        }
    }
    if (ImgUtil::DEBUG) {
        cout << "Hay " << lastLabel << " componentes conectadas" << endl;
    }
    return lastLabel;
}

int ComputeLabels::fillLabelRegion(int posI, int posJ){

    for (int i = max(0, posI - 1); i <= min(posI + 1, h - 1); i++) {
        for (int j = max(0, posJ - 1); j <= min(posJ + 1, w - 1); j++) {
            if ((img[i][j] == 1) && (labelImg[i][j] == 0)) {
                labelImg[i][j] = lastLabel;
                nComponents[lastLabel]++;
                fillLabelRegion(i, j);
            }
        }
    }
    
    return 0;
}

int ComputeLabels::getBiggestLabel(){
    int max, label;
    
    if (img == 0){
        cout << "error, no hay imagen etiquetada" << endl;
        return -1;
    }
    else{
        max = -100000;
        label = 0;
        for (int i = 1; i <= lastLabel; i++){
            if (nComponents[i] > max){
                max = nComponents[i];
                label=i;
            }
        }
    }
    
    return label;
}