/* 
 * \file        ComputeLabels.h
 * \author      Juan Cerrón González (juan.cerron@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * Header file for ComputeLabels  class
 *
 * 
 */


#ifndef COMPUTELABELS_H
#define	COMPUTELABELS_H

using namespace std;

class ComputeLabels{
public:
    
    /*Default constructor*/
    ComputeLabels();
    
    /*Default deconstructor*/
    ~ComputeLabels();
    
    /**
     * It calculates the connected components of a given image.
     * \param imagen binary image to calculate the components
     * \param h image's height
     * \param w image's width
     * \return number of labels found
     */
    int calculateLabels(int **imagen, int h, int w);
    
    /**
     * It calculates recursively the connected components of the pixel [posI][posJ]
     * and add to the label
     * \param posI vertical index of the pixel
     * \param posJ horizontal index of the pixel
     */
    int fillLabelRegion(int posI, int posJ);
    
    /**
     * It returns the label corresponding to the biggest area of the connected
     * components
     * \return number of label
     */
    int getBiggestLabel();
    
    /**
     * It returns the calculated labeled image
     * \return labeled image
     */
    int **getLabeledImage();
    
private:
    int *nComponents;           //!< Array with the number of components found for each label
    int lastLabel;              //!< Last label of the nComponents array
    int **labelImg;             //!< Image with the index of the correspondent label.
    
    int **img;                  //!< Image to calculate the connected components
    int h;                      //!< labelImg's height
    int w;                      //!< labelImg's width
};

inline int **ComputeLabels::getLabeledImage(){ return labelImg; }

#endif	/* COMPUTELABELS_H */

