/* 
 * \file        EjecutarMetodo.h
 * \author      Mikel Galar Idoate (mikel.galar@unavarra.es)
 * \author      Juan Cerrón González (juan.cerron@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * The definition of EjecutarMetodos methods
 *
 * 
 */

#ifndef EJECUTARMETODO_H
#define	EJECUTARMETODO_H

#include <sys/types.h>
#include <dirent.h>
#include <errno.h>

#include "ImgUtil.h"
#include "MemUtil.hpp"

const int MaxValNom = 20;

using namespace std;

class EjecutarMetodo{
    
public:
    
    EjecutarMetodo();
    ~EjecutarMetodo();
    
    /**
     * It indicates whether the argument argv[1] is a directory or a fingerprint
     * \param dir directory path or fingerprint
     * \param files vector with the names of every file that represents a fingerprint
     * \return -1 if directory not found
     */
    int getdir(string dir, vector<string> &files);
    
    /**
     * It indicates the database being processed
     * \param dir
     * \return -1 if database not found in the name of the directori
     */
    int getDB(string dir);
    
    /**
     * It prints the feature vector and completes dataset headers' variables
     * \param fv feature vector
     * \param n feature vector's dimension
     * \param file fingerprint file name
     * \param metodo the used method name
     * \param out file to write the output in
     */
    void printFeatureVector(float *fv, int n, string file, char *metodo, ostream &out = std::cout);
    
    /**
     * It prints the dataset header from the variables completed in printFeatureVector
     * \param n feature vector's dimension
     * \param metodo the used method name
     * \param out file to write the output in
     */
    void printDataSetHeader(int n, char *metodo, ostream &out);
    
    /**
     * It orders a given vector in growing order 
     * \param v vector
     */
    void orderVector(int *v);
    
    /**
     * It searches an element in a vector and returns -1 if it hasn't been found
     * \param v vector
     * \param valor value to be searched
     * \return -1 if not found, 0 otherwise
     */
    int searchElto(int *v, int valor);
    
    /**
     * It copies the data in the temporal file to the final data file under the header
     * \param out final data file
     */
    void copyDataSet(ostream &out);
    
    /**
     * It runs the method indicated with every fingerprint of the directory.
     * \param direc directory
     * \param metodo method
     * \param salida file to write the output in
     * \param ajustar indicates whether to use Sfinge adjust or redimensioned image
     * \param realzar indicates whether to take the enhancement image or not. 
     * \param fichSalidaOk bool to identify whether there is an output file
     * \param debug bool to indicate whether to execute debug or not
     */
    void ejecutar(char* direc, char* metodo, char* salida, bool ajustar, bool realzar, bool fichSalidaOk, bool debug);
    
    /**
     * It extracts the type of fingerprint from the file's name (for Esfinge fingerprints)
     * \param fileName File's name
     * \return a number that indicates the type of fingerprint (1 arch, 2 tented-arch, 3 left loop, 4 right loop, 5 whorl)
     */
    int extractFingerprintType(string fileName);
    
public:
    float *maximos;         //<!Array with maximum values of every attribute
    float *minimos;         //<!Array with minimum values of every attribute
    string fTemporal;       //<!Temporal file name for writing the header before all dates
    
    int *tipos;             //<!Array that indicates the type of every attribute
    int **valoresNom;       //<!Vector with all possible nominal values
};

#endif	/* EJECUTARMETODO_H */

