/* 
 * \file        GHMFiltering.cpp
 * \author      Carlos Lopez Molina (carlos.lopez@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * Implementation of the GHMFiltering class
 *
 * 
 */


#include "MemUtil.hpp"
#include "ImgUtil.h"
#include "MathUtil.h"


#include "GHMFiltering.h"

namespace GHMFiltering {
 
    float *computeGHMImage(float **imgFiltered, int h, int w, float **Mu, float **Mv,float **energy, float **magn, float support, float sigma, float lambda){

        //float lambda=lambdaLinWangFE;
        int supportHalfSize=support;
        //float sigma=ghmSigmaLinWangFE;

        //Create support vector
        //cout <<"\t\t1.1.1 - Create support vector (support half size=["<<supportHalfSize<<"]."<<endl;
        int xMin=-supportHalfSize;
        int xMax=supportHalfSize;
        float step=1;

        int xLength=(xMax-xMin)*((int)(1/step))+1;
        float *xValues=MemUtil::createVector<float>(xLength,0);

        for (int i=0; i<xLength; i++){ xValues[i]=xMin+ step*i; }

        //Create filters
        //cout <<"\t\t1.1.2 - Create filters"<<endl;

        float **ghmFilter10=MemUtil::create2DMatrix<float>(xLength,xLength,0);
        float **ghmFilter01=MemUtil::create2DMatrix<float>(xLength,xLength,0);
        float **ghmFilter30=MemUtil::create2DMatrix<float>(xLength,xLength,0);
        float **ghmFilter03=MemUtil::create2DMatrix<float>(xLength,xLength,0);

        computeGHFilter(&ghmFilter10,xValues,xLength,1,xValues,xLength,0,sigma);
        computeGHFilter(&ghmFilter01,xValues,xLength,0,xValues,xLength,1,sigma);
        computeGHFilter(&ghmFilter30,xValues,xLength,3,xValues,xLength,0,sigma);
        computeGHFilter(&ghmFilter03,xValues,xLength,0,xValues,xLength,3,sigma);

        MemUtil::freeMatrix(&xValues);
        //Convolve with filters
        //cout <<"\t\t1.1.3 - Convolve"<<endl;    

        float **M10=MemUtil::create2DMatrix<float>(h,w,0);
        float **M01=MemUtil::create2DMatrix<float>(h,w,0);
        float **M30=MemUtil::create2DMatrix<float>(h,w,0);
        float **M03=MemUtil::create2DMatrix<float>(h,w,0);



        float imVal;
        for (int i=0;i<h; i++){
            for (int j=0;j<w; j++){

                for (int u=-supportHalfSize;u<=supportHalfSize; u++){
                    for (int v=-supportHalfSize;v<=supportHalfSize; v++){

                        //Policy-> boundaries replication
                        imVal=imgFiltered[min(h-1,max(0,i+u))][min(w-1,max(0,j+v))];

                        M10[i][j]+= imVal* -ghmFilter10[u+supportHalfSize][v+supportHalfSize];
                        M01[i][j]+= imVal* ghmFilter01[u+supportHalfSize][v+supportHalfSize];
                        M30[i][j]+= imVal* -ghmFilter30[u+supportHalfSize][v+supportHalfSize];
                        M03[i][j]+= imVal* ghmFilter03[u+supportHalfSize][v+supportHalfSize];

                    }
                }

            }
        }


        float *maxmin=MemUtil::createVector<float>(3);
        maxmin[0]=-100000;
        maxmin[1]=100000;
        maxmin[2]=0;
        
        float maxEnergy=0;

        for (int i=0;i<h; i++){
            for (int j=0;j<w; j++){
                Mv[i][j]= lambda*M10[i][j]+(1-lambda)*M30[i][j];
                Mu[i][j]= lambda*M01[i][j]+(1-lambda)*M03[i][j];
                magn[i][j]=sqrt((Mv[i][j]*Mv[i][j])+(Mu[i][j]*Mu[i][j]));
                maxmin[2]+=magn[i][j];
                energy[i][j]=(M10[i][j]*M10[i][j])+(M01[i][j]*M01[i][j])+(M30[i][j]*M30[i][j])+(M03[i][j]*M03[i][j]);
                //energy[i][j]=M10[i][j])M01[i][j]+M30[i][j]+M03[i][j];
                if (energy[i][j]>maxEnergy){
                    maxEnergy=energy[i][j];
                }
                if (magn[i][j]>maxmin[0]){maxmin[0]=magn[i][j];}
                if (magn[i][j]<maxmin[1]){maxmin[1]=magn[i][j];}
            }
        }
        for (int i=0;i<h; i++){
            for (int j=0;j<w; j++){
                energy[i][j]=energy[i][j]/maxEnergy;
                
            }
        }

        maxmin[2]=maxmin[2]/(h*w);
        
        if (ImgUtil::DEBUG){
            string filename1="M10";
            ImgUtil::writeGrayScaleImage(M10,h,w,filename1,5);
            string filename2="M01";
            ImgUtil::writeGrayScaleImage(M01,h,w,filename2,5);
            string filename3="M30";
            ImgUtil::writeGrayScaleImage(M30,h,w,filename3,5);
            string filename4="M03";
            ImgUtil::writeGrayScaleImage(M03,h,w,filename4,5);
            string filename5="Mu";
            ImgUtil::writeGrayScaleImage(Mu,h,w,filename5,5);
            string filename6="Mv";
            ImgUtil::writeGrayScaleImage(Mv,h,w,filename6,5);
            string filename7="magn";
            ImgUtil::writeGrayScaleImage(magn,h,w,filename7,1);            

        }


        
        MemUtil::freeMatrix(&ghmFilter10);
        MemUtil::freeMatrix(&ghmFilter01);
        MemUtil::freeMatrix(&ghmFilter30);
        MemUtil::freeMatrix(&ghmFilter03);
        
        MemUtil::freeMatrix(&M10);
        MemUtil::freeMatrix(&M01);
        MemUtil::freeMatrix(&M30);
        MemUtil::freeMatrix(&M03);
        
        return maxmin;
        
    }

    float *computeHermitePoly(float **y,int order,float *xValues, int length){

        if (order<0){
            cout <<" Warning at WangFE::computeHermitePoly-> Order should not be under 0."<<endl;
        }

        float *h=MemUtil::createVector<float>(order+1,0);

        recursiveHermite(&h,order);

        for (int val=0; val<length; val++){

            for (int expnt=0; expnt<order+1; expnt++){
                float aux=h[order-expnt]*(float) pow(xValues[val],((float)expnt));
                (*y)[val]=(*y)[val]+ aux;
            }

        }

        MemUtil::freeMatrix(&h);

    }

    void recursiveHermite(float **polynomial,int order){


        if (order==0){
            (*polynomial)[0]=1;
        }
        else if (order==1){
            (*polynomial)[0]=2;
            (*polynomial)[1]=0;
        }
        else{

            float *auxH1=MemUtil::createVector<float>(order+1,0);            
            float *prevPol=MemUtil::createVector<float>(order,0);

            recursiveHermite(&prevPol,order-1);   

            for (int i=0;i<order;i++){
                auxH1[i]=2*prevPol[i];
            }

            float *auxH2=MemUtil::createVector<float>(order+1,0);  
            float *prevPol2=MemUtil::createVector<float>(order-1,0);

            recursiveHermite(&prevPol2,order-2);


            for (int i=2; i<order+1; i++){
                auxH2[i]=2*(order-1)*prevPol2[i-2];
            }


            //h = h1 - h2;
            for (int i=0;i<order+1;i++){
                (*polynomial)[i] = auxH1[i] - auxH2[i];

            }
            
            MemUtil::freeMatrix(&auxH1);
            MemUtil::freeMatrix(&auxH2);
            MemUtil::freeMatrix(&prevPol);
            MemUtil::freeMatrix(&prevPol2);

        }
    }

    void computeGHFilter(float ***filter, float *vValues, int vLength, int vOrder,float *hValues, int hLength, int hOrder,float sigma){

        //cout <<"\t\t starting computeGHFilter"<<endl;
        //Adapting values to be value/sigma
        float* vValuesCond=MemUtil::createVector<float>(vLength,0);
        float* hValuesCond=MemUtil::createVector<float>(hLength,0);

        for (int i=0;i<vLength; i++){ vValuesCond[i]= vValues[i]/sigma;}
        for (int i=0;i<hLength; i++){ hValuesCond[i]= hValues[i]/sigma;}


        //cout <<"\t\t starting computeGHFilter2"<<endl;
        float* vPolyValues=MemUtil::createVector<float>(vLength,0);
        computeHermitePoly(&vPolyValues,vOrder,vValuesCond,vLength);


        float* hPolyValues=MemUtil::createVector<float>(hLength,0);
        computeHermitePoly(&hPolyValues,hOrder,hValuesCond,hLength);


        //Calculate Gaussian filter

        //cout <<"\t\t starting computeGHFilter3"<<endl;
        float **GauFil=MemUtil::create2DMatrix<float>(vLength,hLength,0);
        float midGauFilI=vLength/2;
        float midGauFilJ=hLength/2;
        float sigmaSquare=sigma*sigma;

        for (int i=0; i<vLength; i++){
            for (int j=0; j<hLength; j++){
                float d=(i-midGauFilI)*(i-midGauFilI)+(j-midGauFilJ)*(j-midGauFilJ);
                GauFil[i][j]=exp(-(d/(2*sigmaSquare)));
            }
        }

        //Generate the GHMs
        float acc=0;
        for (int i=0; i<vLength; i++){
            for (int j=0; j<hLength; j++){
                (*filter)[i][j]=GauFil[i][j]*vPolyValues[i]*hPolyValues[j];
                acc+=abs((*filter)[i][j]);
            }
        }

        //Normalize
        for (int i=0; i<vLength; i++){
            for (int j=0; j<hLength; j++){
                (*filter)[i][j]=(*filter)[i][j]*2/acc;
            }
        }
        
        //cout <<"\t\t starting computeGHFilter5"<<endl;

        //Free memory
        MemUtil::freeMatrix(&GauFil);
        MemUtil::freeMatrix(&vPolyValues);
        MemUtil::freeMatrix(&hPolyValues);
        MemUtil::freeMatrix(&vValuesCond);
        MemUtil::freeMatrix(&hValuesCond);

        return;
    }
    
    
    //int startI, int endI, int startJ, int endJ, 
    
    void computeCoherence(float **C, int h, int w, float **xCos, float **ySin,int halfWindowSize){
        int *ySpan=MemUtil::createVector<int>(2);
        ySpan[0]=0;
        ySpan[1]=h-1;
        int *wSpan=MemUtil::createVector<int>(2);
        wSpan[0]=0;
        wSpan[1]=w-1;
        
        computeCoherence(C,h,w,xCos,ySin,halfWindowSize,ySpan,wSpan);
        
        MemUtil::freeMatrix(&ySpan);
        MemUtil::freeMatrix(&wSpan);
    }
    
    
    void computeCoherence(float **C, int h, int w, float **xCos, float **ySin,int halfWindowSize, int *ySpan, int *wSpan){
    
    //cout << "\t\t\t (inside compute coherence, ("<<startI<<":"<<endI<<","<<startJ<<":"<<endJ<<")."<<endl;
    
    //cout << "\t\t\t (inside compute coherence, (halfsize = "<<halfWindowSize<<")."<<endl;
        
    //this->C=MemUtil::create2DMatrix<float>(this->h,this->w,0);
        
    float maxC=0;
    
    
    
    for (int i=ySpan[0]; i<ySpan[1]; i++){
        
        int fromU=max(0,i-halfWindowSize);
        int toU=min(h-1,i+halfWindowSize);
        
        //cout << i<<endl;
        
        
        
        for (int j=wSpan[0]; j<wSpan[1]; j++){
            
            
            int fromV=max(0,j-halfWindowSize);
            int toV=min(w-1,j+halfWindowSize);
            
            int numOfElementsInTheWindow=((1+toU-fromU)*(1+toV-fromV));
            
            //cout <<"\t ("<<fromU<<"->"<<toU<<") y ("<<fromV<<"->"<<toV<<") sum-> "<<numOfElementsInTheWindow<<endl;
            
            //cout <<"xcos[0][0]="<<xCos[0][0]<<endl;
            float** localVectorList=MemUtil::create2DMatrix<float>(numOfElementsInTheWindow,2);
            
            int vectorCounter=0;
            
            for (int u=fromU; u<=toU; u++){
                for (int v=fromV; v<=toV; v++){
                    
                    localVectorList[vectorCounter][0]=xCos[u][v];
                    localVectorList[vectorCounter][1]=ySin[u][v];
                    vectorCounter++;
                    //cout <<"\t\t"<<vectorCounter<<endl;
                }
            }
            
            //cout <<"about to obtain the covariances"<<endl;
            float **covMatrix=MathUtil::makeCovariancesMatrix(localVectorList,numOfElementsInTheWindow,2);
            //cout <<"covariances obtained"<<endl;
            float *eigenvalues=MathUtil::getEigenvalues(covMatrix,2);
            
            //cout <<"eigenvalues obtained"<<endl;
            
            float eigOne=max(abs(eigenvalues[0]),abs(eigenvalues[1]));
            float eigTwo=min(abs(eigenvalues[0]),abs(eigenvalues[1]));
            
            C[i][j]= (eigOne-eigTwo)/(eigOne+eigTwo);
            
            MemUtil::freeMatrix(&localVectorList);
            MemUtil::freeMatrix(&covMatrix);
            MemUtil::freeMatrix(&eigenvalues);
            
            if (C[i][j]> maxC){
                maxC=C[i][j];
            }
                
        }
            
    }
    
    //cout << "Exiting"<<endl;
    
    for (int i=ySpan[0]; i<ySpan[1]; i++){
        for (int j=wSpan[0]; j<wSpan[1]; j++){
            C[i][j]=C[i][j]/maxC;
        }
    }
    
}
}