/* 
 * \file        GaborFilter.h
 * \author      Mikel Galar Idoate (mikel.galar@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * Header file for GaborFilter  class
 *
 * 
 */

#ifndef GABORFILTER_H
#define	GABORFILTER_H

using namespace std;

class GaborFilter {
public:
    
    /* Default constructor */
    GaborFilter();
    
    /**
     * It creates the gabor filter with the parameters below.
     * \param nFilters
     * \param gabor_frequency:  ( 1 / K ) where K is the interridge distance
     * \param leung_factor:     It is a constant that adjusts yprima^2
     * \param gabor_dx:         Determines the bandwidth of the gabor filters
     * \param gabor_dy:         Determines the bandwidth of the gabor filters
     * \param gaborSupportX     Support of gabor filters
     * \param gaborSupportY     Support of gabor filters
     */
    GaborFilter(int nFilters, float gabor_frequency, double leung_factor, double gabor_dx, double gabor_dy,int gaborSupportX, int gaborSupportY);
    
    /* Default destructor */
    ~GaborFilter();
    
    /**
     * It returns the number of filters constructed (in gaborFilters)
     * \return Number of Gabor filters in gaborFilters
     */
    int getnFilters()const;
    
    /**
     * It filters an image with all the Gabor Filters, creating a new image
     * for each one of the Gabor Filters considered
     * \param image the image to be filtered
     * \param h height of the image
     * \param w width of the image
     * \return the filtered images (one for each orientation) 
     * with size  h - (GABOR_SUPPORT - 1) *  w - (GABOR_SUPPORT - 1)
     */
    float ***filterImage(float **image, int h, int w);
    
    
private:
    float *orientations;        //!< Orientations used in each Gabor filter (there are nFilters orientations) equally distributed
    int nFilters;               //!< Number of filters to be constructed
    
    float ***gaborFilters;      //!< The Gabor filters to be used
        
    int gaborSupportX;           //!< Support of the Gabor Filter
    
    int gaborSupportY;           //!< Support of the Gabor Filter
    
    bool ***gaborMask;          //!< The mask for values lower than 0.05
    
    float max;
    float min;
};


inline int GaborFilter::getnFilters() const{ return nFilters;}

#endif	/* GABORFILTER_H */

