/* 
 * \file        ImgUtil.h
 * \author      Mikel Galar Idoate (mikel.galar@unavarra.es)
 * \author      Juan Cerrón González (juan.cerron@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * The definition of ImgUtil namespace
 *
 * 
 */

#ifndef IMGUTIL_H
#define	IMGUTIL_H

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cmath>
#include <vector>

#include "MemUtil.hpp"
#include "../include/Constants.h"

#ifdef __cplusplus
extern "C" {
#endif

#include "include/pca.h"
#include "imgdecod.h"
// needed for pcasys
#ifndef _debug
    extern int debug;
#endif
    
#ifdef __cplusplus
}
#endif

const int TEST_1 = 0;                 // 1 for DEBUG mode!

const int colorBackground = 150;

using namespace std;


namespace ImgUtil{
    
    extern bool DEBUG; // = 1;
    extern float fAdjust;// = 1;
    extern bool doAdjust;// = true;
    extern bool isSfinge;// = false;
    
    void setDebug(bool dbg);
    void setfAdjust(float fAdjust);
    void setDoAdjust(bool doAdjust);
    void setIsSfinge(bool isSfinge);
    
    int writeGrayScaleImageCore(float **image, int h, int w, string fileName, int coreX, int coreY);
    
/**
     * It writes a gray scale image image to a file
     * \param image image to be writen (float in [0 255])
     * \param nRow number of rows
     * \param nCol number of columns
     * \param scale factor for the intensity of each pixel
     * \param fileName file name of the image (without extension)
     * \return -1 if an error has ocurred. 0 otherwise
     */
    int writeGrayScaleImage(float **image, int nRow, int nCol, string fileName, float scale, bool **masc = 0, int blockW = 0);
    
    int writeGrayScaleImage(int **image, int nRow, int nCol, string fileName, float scale);
    
    int writeColorImage(int ***image, int nRow, int nCol, string fileName);

    int writeSPImage(float **image, int nRow, int nCol, vector<float> deltas, vector<float> cores, int blockSize, string fileName);
    int writeSPImage(float **image, int nRow, int nCol, vector<int> deltas, vector<int> cores, int blockSize, string fileName);
    
    int writeSPImage(float **image, int nRow, int nCol, int *deltas, int nDeltas, int *cores, int nCores, string fileName);
    
    int writeSPImage(float **image, int nRow, int nCol, int **deltas, int nDeltas, int **cores, int nCores, string fileName);
    
    /**
     * It writes a gray scale image image to a file
     * \param image image to be writen (unsigned char in [0 255])
     * \param nRow number of rows
     * \param nCol number of columns
     * \param scale factor for the intensity of each pixel
     * \param fileName file name of the image (without extension)
     * \return -1 if an error has ocurred. 0 otherwise
     */
    int writeGrayScaleImage(unsigned char **image, int nRow, int nCol, string fileName);
        
    /**
     * It writes a gray scale image image to a file
     * \param image image to be writen (using integers)
     * \param nRow number of rows
     * \param nCol number of columns
     * \param scale factor for the intensity of each pixel
     * \param fileName file name of the image (without extension)
     * \return -1 if an error has ocurred. 0 otherwise
     */
    int writeGrayScaleImage(int **image, int nRow, int nCol, string fileName);
    
    /**
     * It writes a gray scale image image to a file
     * \param image image to be writen (using booleans)
     * \param nRow number of rows
     * \param nCol number of columns
     * \param scale factor for the intensity of each pixel
     * \param fileName file name of the image (without extension)
     * \return -1 if an error has ocurred. 0 otherwise
     */
    int writeGrayScaleImage(bool **image, int nRow, int nCol, string fileName);
    
    void writeImageAndTrace(float **gImage, int **CI, string fileName, int w, int h, int Ws, int wCI, int hCI);
    
    /**
     * It is used in getOrientationsImagePicture function
     */
    void xy_to_dmt(float **x, float **y, float **dg, float **mag, int w, int h, float *top_ret);
    
    /**
     * It is used in getOrientationsImagePicture function
     */
    void dmt_to_pic(float **dg, float **mag, int aw, int ah, float top,  unsigned char *pic, int w, int h);
    
    /**
     * It obtained the orientation image from the sin and cos (avOrX, avOrY) and a given size (wIn, wOut)
     * \return 
     */
    unsigned char *getOrientationImagePicture(float **avOrX, float **avOrY, int wIn, int hIn, int *wOut, int *hOut);
   
    /**
     * It writes the orientation image.
     * \param X
     * \param Y
     * \param wOr
     * \param hOr
     * \param fileName
     * \return 
     */
    int writeOrImg(float **X, float **Y, int wOr, int hOr, string fileName);
    
    /**
     * It compute SOBEL to calculate the gradients in horizontal and vertical directions.
     * \param img image to calculate the gradient
     * \param h image's height
     * \param w image's width
     * \param gradX gradient in horizontal direction
     * \param gradY gradient in vertical direction
     * \param grad gradient magnitud combination of gradX and gradY
     * \return two array elements with the max and the min amplitude
     */
    float *computeSobel(float **img, int h, int w,float **gradX, float **gradY, float **grad);
    
    void computeSobelGradients(float **img, int h, int w, float ***gradX, float ***gradY, float ***grad);
    
    float **computeOrientationsFromGradients(float **gradX, float **gradY, int h, int w, int blockW, float ***xCos, float ***ySin, int *orH, int *orW, bool normalize, float **coherence = 0);
    
    /**
     * It binarizes an image according to a given umbral 
     * if img < umbral then 0
     * else 1
     * \param img image to be binarized
     * \param h image's height
     * \param w image's width
     * \param umbral 
     * \return binarized image
     */
    int **binarizeImg(float **img, int h, int w, int umbral);
    
    /**
     * It computes the histogram of the image.
     * \param image to be processed
     * \param nRow image's height
     * \param nCol image's width
     * \return the histogram of the image
     */
    float *computeHistogram(float **img, int nRow, int nCol);
    
    /**
     * It segments the image at a certain threshold.
     * \param image to be segmented
     * \param nRow image's height
     * \param nCol image's width
     * \param threshold threshold to be used in the image segmentation process
     * \return the segmented image
     */
    float **thresholdImage(float **image, int nRow, int nCol, float threshold);
    
    /**
     * It returns an image filtered with a masc.
     * \param image image to be filtered
     * \param h image's height
     * \param w image's width
     * \param masc masc to be used
     * \param hMasc masc's height
     * \param wMasc masc's width
     * \return filtered image
     */
    template<typename T>T **getModifiedImgMinMax(T **image, int h, int w, T **masc, int hMasc, int wMasc){
        int card;
        int mitadX = hMasc/2;
        int mitadY = wMasc/2;

        T **imgFiltered = MemUtil::create2DMatrix<T>(h,w);
        //Recorre la imagen
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {
                card=0;

                //Para cada punto de la imagen, calculamos el valor al aplicar la máscara
                for (int u = max(0, i - mitadX); u < min(h, (int)(i + ceil((double)hMasc/2))); u++){
                    for (int v = max(0, j - mitadY); v < min(w, (int)(j + ceil((double)wMasc/2))); v++){
                        card++;
                        imgFiltered[i][j] += image[u][v]*masc[u-(i-mitadX)][v-(j-mitadY)];
                    }
                }
                //cout << "Pixel ["<<i<<"]"<<"["<<j<<"] = "<<imgFiltered[i][j]<< " (card = "<<card<<")"<<endl;
                imgFiltered[i][j] = imgFiltered[i][j]/(float)card;
            }
        }
        
        return imgFiltered;
    }
    
    /**
     * It returns an image filtered with a masc.
     * \param image image to be filtered
     * \param h image's height
     * \param w image's width
     * \param masc masc to be used
     * \param hMasc masc's height
     * \param wMasc masc's width
     * \return filtered image
     */
    template<typename T>T **getModifiedImgExtended(T **image, int h, int w, T **masc, int hMasc, int wMasc){
        int card;
        int mitadX = hMasc/2;
        int mitadY = wMasc/2;
        
        T **imgExtended = MemUtil::create2DMatrix<T>(h,w);
        //Recorre la imagen
        for (int i = mitadX; i < h + mitadX; i++) {
            for (int j = mitadY; j < w + mitadY; j++) {
                card=0;

                //Para cada punto de la imagen, calculamos el valor al aplicar la máscara
                for (int u = 0; u < hMasc; u++){
                    if (((i - mitadX + u) >= mitadX) && ((i + u) < h + (2 * mitadX)) ){
                        for (int v = 0; v < wMasc; v++){
                            if (((j - mitadY + v) >= mitadY) && ((j + v) < w + (2 * mitadY)) ){
                                card++;
                                imgExtended[i-mitadX][j-mitadY] += image[(i-mitadX)+u-mitadX][(j-mitadY)+v-mitadY]*masc[u][v];
                            }
                        }
                    }
                }
                //cout << "Pixel ["<<i<<"]"<<"["<<j<<"] = "<<imgExtended[i-mitadX][j-mitadY]<< " (card = "<<card<<")"<<endl;
                imgExtended[i-mitadX][j-mitadY] = imgExtended[i-mitadX][j-mitadY]/(float)card;
            }
        }  
        return imgExtended;
    }
    
    /**
     * It returns an image filtered with a masc.
     * \param image image to be filtered
     * \param h image's height
     * \param w image's width
     * \param masc masc to be used
     * \param hMasc masc's height
     * \param wMasc masc's width
     * \return filtered image
     */
    template<typename T>T **getModifiedImgSinBorde(T **image, int h, int w, T **masc, int hMasc, int wMasc){
        int card;
        int mitadX = hMasc/2;
        int mitadY = wMasc/2;
        
        T **imgExtended = MemUtil::create2DMatrix<T>(h,w);
        //Recorre la imagen
        for (int i = mitadX; i <= h - (int)(ceil((double)hMasc/2)); i++) {
            for (int j = mitadY; j <= w - (int)(ceil((double)wMasc/2)); j++) {
                card=0;

                //Para cada punto de la imagen, calculamos el valor al aplicar la máscara
                for (int u = 0; u < hMasc; u++){
                    for (int v = 0; v < wMasc; v++){
                        card++;
                        imgExtended[i][j] += image[(i-mitadX)+u][(j-mitadY)+v]*masc[u][v];
                    }
                }
                //cout << "Pixel ["<<i<<"]"<<"["<<j<<"] = "<<imgExtended[i-mitadX][j-mitadY]<< " (card = "<<card<<")"<<endl;
                imgExtended[i][j] = imgExtended[i][j] / (float)card;
            }
        }  
        return imgExtended;
    }
    
    /**
     * It returns an image filtered with a normalized masc.
     * \param image image to be filtered
     * \param h image's height
     * \param w image's width
     * \param masc masc to be used
     * \param hMasc masc's height
     * \param wMasc masc's width
     * \return filtered image
     */
    template<typename T>T **getFilteredImg(T **image, int h, int w, T **masc, int hMasc, int wMasc){
        int mitadX = hMasc/2;
        int mitadY = wMasc/2;

        T **imgFiltered = MemUtil::create2DMatrix<T>(h,w);
        //Recorre la imagen
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {

                //Para cada punto de la imagen, calculamos el valor al aplicar la máscara
                for (int u = max(0, i - mitadX); u < min(h, (int)(i + ceil((double)hMasc/2))); u++){
                    for (int v = max(0, j - mitadY); v < min(w, (int)(j + ceil((double)wMasc/2))); v++){
                        imgFiltered[i][j] += image[u][v]*masc[u-(i-mitadX)][v-(j-mitadY)];
                        if (i < mitadX || i >= h-mitadX || j < mitadY || j >= w-mitadY )
                            imgFiltered[i][j] = image[i][j];
                    }
                }
            }
        }
        
        return imgFiltered;
    }
    
    template<typename T>T **getFilteredImgReduced(T **image, int h, int w, T **masc, int hMasc, int wMasc){
        int mitadX = hMasc / 2;
        int mitadY = wMasc / 2;

        T **imgFiltered = MemUtil::create2DMatrix<T>(h - hMasc + 1, w - wMasc + 1);
        //Recorre la imagen
        for (int i = mitadX; i < h - mitadX; i++) {
            for (int j = mitadY; j < w - mitadY; j++) {

                //Para cada punto de la imagen, calculamos el valor al aplicar la máscara
                for (int u = i - mitadX; u < i + mitadX + 1; u++){
                    for (int v = j - mitadY; v < j + mitadY + 1; v++){
                        imgFiltered[i - mitadX][j - mitadY] += image[u][v] * masc[u - (i - mitadX)][v - (j - mitadY)];
                    }
                }
            }
        }
        
        return imgFiltered;
    }
    
    bool **imDilate(bool **img, int orH, int orW);
    
    bool **imFill(bool **img, int orH, int orW, int nRounds = 1);
    
    bool **imErode(bool **img, int orH, int orW);
    
    void smoothOrientations(float **O, float ***xCos, float ***ySin, int orH, int orW, int Wsmooth, bool normalizeLength = true, bool ***segmentation = 0);
    
    void setCurrentFileID(string id);
    
    void setCurrentDB(string id);
    
    string makeTestFileName(string infoName);
    
    string makeTest2FileName(string infoName);
    
    string makeTest3FileName(string infoName);
    
    string makeTest4FileName(string infoName);
    
    /**
     * It fills the border of an image with the given color
     * \param imagen image to fill border
     * \param h image's height
     * \param w image's width
     * \param iniRow row image starting index
     * \param iniCol column image starting index
     * \param endRow row image ending index
     * \param endCol column image ending index
     * \param color border color
     */
    void fillBorderColor(float **imagen, int h, int w, int iniRow, int endRow, int iniCol, int endCol, float color);
    
    /**
     * It completes the border of an imagen by extending the border of other image.
     * \param imgExtended image to fill borders
     * \param fingerImg image to get the original pixel border value
     * \param filterRadioI border in the begining and ending of every row
     * \param filterRadioJ border in the begining and ending of every column
     * \param h image's height
     * \param w image's width
     */
    void fillBorderExtending(float **imgExtend, float **fingerImg, int filterRadioI, int filterRadioJ, int h, int w);
    
    
//private:
        
    extern string currentFileID;
    extern string currentDB;
    
    float **createGaussianKernel(float sigma, int *gWout) ;
    float *create1DGaussianKernel(float sigma, int *gWout);
    float **createGaussianKernel(float sigmaH, float sigmaW, int h, int w);

    float **downSampleImg(float **img, int h, int w, int *hOut, int *wOut, int scale);

    
    float otsuThresholding(int *histogram, float *values, int numValues);
    
    float directionDifference(float a1, float a2);
    
    float directionMean(float a1, float a2);

}


#endif	/* UTIL_H */

