 /* 
 * \file        LlamaMetodos.cpp
 * \author      Mikel Galar Idoate (mikel.galar@unavarra.es)
 * \author      Juan Cerrón González (juan.cerron@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * The implementation of LlamaMetodos namespace
 *
 * 
 */

#include "LlamaMetodos.h"


#include "MemUtil.hpp"
#include "FingerprintImg.h"

#include "NewPoincareSingularPoints.h"
#include "ImageEnhancementHong.h"
#include "OrientationFFTPark.h"
#include "ComplexFiltersLiu.h"
#include "FingerCode.h"
#include "Zhang.h"
#include "CovarianceBasedCoreDetector.h"

#include "OrientationFieldMei.h"
#include "SymVarReferencePointLe.h"
#include "SPAnglesNyongesa.h"
#include "SPClassificationKaru.h"
#include "HongFeatureExtractor.h"
#include "DirectionImagePartitioningCapelli.h"
#include "OrientationEnhancementCappelli.h"
#include "MultiClassifierCappelli.h"
#include "ComplexFiltersNilssonBigun.h"

#include "FeedbackBasedLineDetector.h"
#include "OrientationReconstructionLi.h"
#include "XuejunTanFE.h"
#include "LinWangFE.h"

#include "GuardarRealce.h"

LlamaMetodos::LlamaMetodos(){
    if (ImgUtil::DEBUG){
        cout << "Creating LlamaMetodos" << endl;
    }
    fv = 0;
    dim = 0;
    err = 0;
}
LlamaMetodos::LlamaMetodos(string file, bool ajustar, bool realce){
    err = 0;
    
    FingerprintImg *fi = new FingerprintImg();


    err = fi->readFingerprintImage(file);
    if (err != 0)
        cout << "error al leer la huella" << endl;
    ImgUtil::setDoAdjust(ajustar);
    fi->setEnhanceHong(realce);
    
    if ((fi->getH() !=  480) && (fi->getW() != 512))
        ImgUtil::setIsSfinge(true);

    if (ImgUtil::doAdjust){
        ImgUtil::setfAdjust(0.5625);

        pcasys = new PCASYSManager(fi->getW(), fi->getH());
    }
    else
        pcasys = new PCASYSManager();

    delete fi;
    fv = 0;
    dim = 0;
}

LlamaMetodos::~LlamaMetodos(){
    if (ImgUtil::DEBUG)
        cout << "Desctructing LlamaMetodos" << endl;
    if (fv != 0)
        MemUtil::freeMatrix(&fv);
    delete pcasys;
    
    if (ImgUtil::DEBUG)
        cout << "LlamaMetodos destructed" << endl;

}

void LlamaMetodos::clear(){
    MemUtil::freeMatrix(&fv);
    fv = 0;
    dim = 0;
}

int LlamaMetodos::doMethod(string file, string metodo){

    if (metodo == ""){
        cout << "Error, no existe el metodo indicado.\n"
                    "Ejemplos:\n\t<Pcasys>\n\t<HongFe>\n\t<Poincare>\n\t<Jain[R92]>\n\t<Liu[Poincare]>"
                    "\n\t<Leung[Poincare]>\n\t<Zhang[Poincare]>\n\t<Le[Poincare]>\n\t<Capelli[2]>\n\t<Nyongesa>"
                    "\n\t<Karu>\n\t<NilssonBigun>\n\t<ShahSastry>\n\t<Li[Poincare]>\n\t<Park21>\n\t<Park105>\n\t<XuejunTan>\n\t"
                    "\n\t<Wang>\n";
        return -1;
    }
    
    if (err != 0)
        return -1;

    if (ImgUtil::fAdjust != 1){            
        if ( metodo ==  "Pcasys"){
            doPCASYS(file);
            return 0;
        }
        else if ( metodo ==  "HongFe") {
            doHongFE(file);
            return 0;
        }
        else if ( metodo ==  "Poincare") 
            cout << "Error, metodo no implementado para huellas distintas de NIST.\n";
        else if ( metodo ==  "JainR92"){ 
            doJAIN(file);
            return 0;
        }
        else if ( metodo ==  "Jain"){ 
            doJAINPoincare(file);
            return 0;
        }
        else if ( metodo ==  "Liu"){
            doLIU(file);
            return 0;
        }
        else if ( metodo ==  "LiuPoincare"){
            doLIU(file);
            return 0;
        }
        else if ( metodo ==  "Leung"){ 
            doLEUNG(file);
            return 0;
        }
        else if ( metodo ==  "LeungPoincare"){
            doLEUNGPoincareSegmentation(file);
            return 0;
        }
        else if ( metodo ==  "Zhang"){
            doZhang(file);
            return 0;
        }
        else if ( metodo ==  "ZhangSegmentation"){ 
            doZhangSegment(file);
            return 0;
        }
        else if ( metodo ==  "Le"){
            doLE(file);
            return 0;
        }
        else if ( metodo ==  "LePoincare"){
            doLESegmentationPoincare(file);
            return 0;
        }
        else if ( metodo ==  "Cappelli"){
            doCappelli(file);
            return 0;
        }
        else if ( metodo ==  "Cappelli2"){
            doCappelli2(file);
            return 0;
        }  
        else if ( metodo ==  "Nyongesa"){
            doNYONGESA(file);
            return 0;
        }
        else if ( metodo ==  "Karu"){
            doKARU(file);
            return 0;
        }
        else if ( metodo == "NilssonBigun"){
            doNilssonBigun(file);
            return 0;
        }
        else if ( metodo == "ShahSastry"){
            doShahSastry(file);
            return 0;
        }
        else if ( metodo == "Li"){
            doLi(file);
            return 0;
        }
        else if ( metodo == "LiPoincare"){
            doLiSegmentationPoincare(file);
            return 0;
        }
        else if ( metodo == "Park21"){
            doPark21(file);
            return 0;
        }
        else if ( metodo == "Park105"){
            doPark105(file);     
            return 0;
        }
        else if ( metodo == "XuejunTan"){
            doXuejunTanFE(file);
            return 0;
        }
        else if ( metodo == "Wang"){
            doWangFE(file);
            return 0;
        }
        else if (metodo == "RejectJainR92"){
            doRejectJain(file);
            return 0;
        }
        else if (metodo == "RejectJain"){
            doRejectJainPoincare(file);
            return 0;
        }
        else if (metodo == "RejectHong"){
            doRejectHong(file);
            return 0;
        }
        else if (metodo == "LiuPoincareJainReject"){         
            doLIUSegmentationPoincareWithJainReject(file);
            return 0;
        }
        else if (metodo == "ZhangJainReject"){     
            doZhangWithJainReject(file);
            return 0;
        }           
        else if (metodo == "AreaReject") { 
                doRejectArea(file);
            return 0;
        }           
        else if (metodo == "LiuPoincareAreaReject"){         
            doLIUSegmentationPoincareWithAreaReject(file);
            return 0;
        }
        else if (metodo == "ZhangAreaReject"){     
            doZhangWithAreaReject(file);
            return 0;
        }   
        else if (metodo == "JainAreaReject"){     
            doJainPoincareWithAreaReject(file);
            return 0;
        }  
        else if (metodo == "KaruAreaReject") {
            doKaruWithAreaReject(file);
            return 0;
        }  
        else if (metodo == "KaruJainReject") {
            doKaruWithJainReject(file);
            return 0;
        } 
        else if (metodo == "Cappelli2JainReject") {
            doCappelli2WithJainReject(file);
            return 0;
        }
        else if (metodo == "GuardarRealce") {
            doGetEnhanced(file);
            return 0;
        }
        else
            cout << "Error, no existe el metodo indicado.\n"
                    "Ejemplos:\n\t<Pcasys>\n\t<HongFe>\n\t<Poincare>\n\t<Jain[R92]>\n\t<Liu[Poincare]>"
                    "\n\t<Leung[Poincare]>\n\t<Zhang[Segmentation]>\n\t<Le[Poincare]>\n\t<Capelli[2]>\n\t<Nyongesa>"
                    "\n\t<Karu>\n\t<NilssonBigun>\n\t<ShahSastry>\n\t<Li[Poincare]>\n\t<Park21>\n\t<Park105>\n\t<XuejunTan>\n\t"
                    "\n\t<Wang>\n\t<RejectJainR92>\n\t<RejectJain>\n\t<RejectHong>\n";
        
        return -1;
    }
    else{            
        if ( metodo == "Pcasys")
            doPCASYS(file);
        else if ( metodo == "HongFe")
            doHongFE(file);
        else if ( metodo == "Poincare")
            doPoincare(file);
        else if ( metodo == "JainR92")
            doJAIN(file);
        else if ( metodo == "Jain")
            doJAINPoincare(file);          
        else if ( metodo == "Liu")
            doLIU(file);
        else if ( metodo == "LiuPoincare")
            doLIUSegmentationPoincare(file);
        else if ( metodo == "Leung")
            doLEUNG(file);
        else if ( metodo == "LeungPoincare")
            doLEUNGPoincareSegmentation(file);
        else if ( metodo == "Zhang")
            doZhang(file);
        else if ( metodo == "ZhangSegmentation")
            doZhangSegment(file);
        else if ( metodo == "Le")
            doLE(file);
        else if ( metodo == "LePoincare")
            doLESegmentationPoincare(file);
        else if ( metodo == "Cappelli")
            doCappelli(file);
        else if ( metodo == "Cappelli2")
            doCappelli2(file);            
        else if ( metodo == "Nyongesa")
            doNYONGESA(file);
        else if ( metodo == "Karu")
            doKARU(file);
        else if ( metodo == "NilssonBigun")
            doNilssonBigun(file);
        else if ( metodo == "ShahSastry")
            doShahSastry(file);
        else if ( metodo == "Li")
            doLi(file);
        else if ( metodo == "LiPoincare")
            doLiSegmentationPoincare(file);
        else if ( metodo == "Park21")
            doPark21(file);
        else if ( metodo == "Park105")
            doPark105(file);      
        else if ( metodo == "XuejunTan")
            doXuejunTanFE(file);
        else if ( metodo == "Wang")
            doWangFE(file);
        else if (metodo == "RejectJainR92")
            doRejectJain(file);
        else if (metodo == "RejectJain")
            doRejectJainPoincare(file);
        else if (metodo == "RejectHong")
            doRejectHong(file);
        else if (metodo == "LiuPoincareJainReject")         
            doLIUSegmentationPoincareWithJainReject(file);
        else if (metodo == "ZhangJainReject")         
            doZhangWithJainReject(file);       
        else if (metodo == "AreaReject")        
                doRejectArea(file);
        else if (metodo == "LiuPoincareAreaReject")         
            doLIUSegmentationPoincareWithAreaReject(file);
        else if (metodo == "ZhangAreaReject")         
            doZhangWithAreaReject(file);   
        else if (metodo == "JainAreaReject")     
            doJainPoincareWithAreaReject(file);
        else if (metodo == "KaruAreaReject")     
            doKaruWithAreaReject(file);
        else if (metodo == "KaruJainReject")     
            doKaruWithJainReject(file);
        else if (metodo == "Cappelli2JainReject")     
            doCappelli2WithJainReject(file);
        else if (metodo == "GuardarRealce"){
            doGetEnhanced(file);
        }
        else{
            cout << "Error, no existe el metodo indicado.\n"
                    "Ejemplos:\n\t<Pcasys>\n\t<HongFe>\n\t<Poincare>\n\t<Jain[R92]>\n\t<Liu[Poincare]>"
                    "\n\t<Leung[Poincare]>\n\t<Zhang[Segmentation]>\n\t<Le[Poincare]>\n\t<Capelli[2]>\n\t<Nyongesa>"
                    "\n\t<Karu>\n\t<NilssonBigun>\n\t<ShahSastry>\n\t<Li[Poincare]>\n\t<Park21>\n\t<Park105>\n\t<XuejunTan>\n\t"
                    "\n\t<Wang>\n\t<RejectJainR92>\n\t<RejectJain>\n\t<RejectHong>\n";
            return -1;
        }

        return 0;
    }
}

int LlamaMetodos::doPCASYS(string file) {
    FingerprintImg *fi = new FingerprintImg();
    fv = fi->run(file, pcasys);
    dim = fi->getDimFV();
    
    delete fi;
}
int LlamaMetodos::doPCASYS2(string file) {
    FingerprintImg *fi = new FingerprintImg();
    fv = fi->run(file, pcasys);
    dim = fi->getDimFV();
    
    delete fi;
}

int LlamaMetodos::doJAIN(string file) {
    FingerCode *fc = new FingerCode();
    fv = fc->run(file, pcasys);
    dim = fc->getDimFV();

    delete fc;
}
int LlamaMetodos::doJAINPoincare(string file) {
    FingerCode *fc = new FingerCode();
    fv = fc->run(file, pcasys, true);
    dim = fc->getDimFV();

    delete fc;
}

int LlamaMetodos::doLEUNG(string file, bool case2) {
    ImageEnhancementHong *ieh = new ImageEnhancementHong();
    fv = ieh->run(file, pcasys, case2);
    dim = ieh->getDimFV();

    delete ieh;
}
int LlamaMetodos::doLEUNGPoincareSegmentation(string file, bool case2) {
    ImageEnhancementHong *ieh = new ImageEnhancementHong();
    fv = ieh->run(file, pcasys, case2, true);
    dim = ieh->getDimFV();

    delete ieh;
}

int LlamaMetodos::doLIU(string file) {
    ComplexFiltersLiu *cfl = new ComplexFiltersLiu();
    fv = cfl->run(file, pcasys);
    dim = cfl->getDimFV();

    delete cfl;
}
int LlamaMetodos::doLIUSegmentationPoincare(string file) {
    ComplexFiltersLiu *cfl = new ComplexFiltersLiu();
    fv = cfl->run(file, pcasys, true);
    dim = cfl->getDimFV();

    delete cfl;
}

int LlamaMetodos::doLE(string file) {
    SymVarReferencePointLe *svrpl = new SymVarReferencePointLe();
    fv = svrpl->run(file, pcasys);
    dim  = svrpl->getDimFV();

    delete svrpl;
}
int LlamaMetodos::doLESegmentationPoincare(string file) {
    SymVarReferencePointLe *svrpl = new SymVarReferencePointLe();
    fv = svrpl->run(file, pcasys, true);
    dim  = svrpl->getDimFV();

    delete svrpl;
}

int LlamaMetodos::doNYONGESA(string file) {
    SPAnglesNyongesa *ssan = new SPAnglesNyongesa();
    fv = ssan->run(file, pcasys);
    dim  = ssan->getDimFV();

    delete ssan;
}

int LlamaMetodos::doKARU(string file) {
    SPClassificationKaru *spck = new SPClassificationKaru();
    fv = spck->run(file, pcasys);
    dim = spck->getDimFV();

    delete spck;
}


int LlamaMetodos::doHongFE(string file) {
    HongFeatureExtractor *hfe = new HongFeatureExtractor();
    fv = hfe->run(file, pcasys);
    dim = hfe->getDimFV();

    delete hfe;
}

int LlamaMetodos::doZhang(string file) {
    Zhang *z = new Zhang();
    fv = z->run(file, pcasys, false);
    dim = z->getDimFV();
    if (fv[0] == -1)        
        cout << "Fingerprint Rejected! Zhang" << endl;

    delete z;
}

int LlamaMetodos::doZhangSegment(string file) {
    Zhang *z = new Zhang();
    fv = z->run(file, pcasys, true);
    dim = z->getDimFV();
    if (fv[0] == -1)        
        cout << "Fingerprint Rejected! Zhang" << endl;

    delete z;
}

int LlamaMetodos::doPoincare(string file) {
    NewPoincareSingularPoints *npsp = new NewPoincareSingularPoints();
    npsp->run(file, pcasys);
    fv = MemUtil::createVector<float>(1);
    fv[0] = -1;
    dim = 1;
    delete npsp;
}

int LlamaMetodos::doCappelli(string file) { 
    DirectionImagePartitioningCapelli *dipc = new DirectionImagePartitioningCapelli();
    fv = dipc->run(file, pcasys);
    dim = dipc->getDimFV();

    delete dipc;
}

int LlamaMetodos::doCappelli2(string file) {
    MultiClassifierCappelli *mcc = new MultiClassifierCappelli();
    fv = mcc->run(file, pcasys);
    dim = mcc->getDimFV();

    delete mcc;
}

int LlamaMetodos::doNilssonBigun(string file) {
    ComplexFiltersNilssonBigun *cfnb = new ComplexFiltersNilssonBigun();
    cfnb->run(file, pcasys);
    fv = MemUtil::createVector<float>(1);
    fv[0] = -1;
    dim = 1;
    delete cfnb;
}

int LlamaMetodos::doShahSastry(string file) {
    FeedbackBasedLineDetector *fdbl = new FeedbackBasedLineDetector();
    fv = fdbl->run(file, pcasys);
    dim = fdbl->getDimFV();
    
    delete fdbl;
}



int LlamaMetodos::doLi(string file) {
    OrientationReconstructionLi *orl = new OrientationReconstructionLi();
    fv = orl->run(file, pcasys);
    dim = orl->getDimFV();
    
    delete orl;
}

int LlamaMetodos::doLiSegmentationPoincare(string file) {
    OrientationReconstructionLi *orl = new OrientationReconstructionLi();
    fv = orl->run(file, pcasys, true);
    dim = orl->getDimFV();
    
    delete orl;
}

int LlamaMetodos::doPark21(string file){
    OrientationFFTPark *offtp = new OrientationFFTPark();
    fv = offtp->run(file, pcasys, false);
    dim = offtp->getDimFV();
    
    delete offtp;
}

int LlamaMetodos::doPark105(string file){
    OrientationFFTPark *offtp = new OrientationFFTPark();
    fv = offtp->run(file, pcasys, true);
    dim = offtp->getDimFV();
    
    delete offtp;
}

int LlamaMetodos::doXuejunTanFE(string file){
    XuejunTanFE *xtfe = new XuejunTanFE();
    fv = xtfe->run(file, pcasys);
    dim = xtfe->getDimFV();
    
    delete xtfe;
}

int LlamaMetodos::doWangFE(string file){
    LinWangFE *lwfe = new LinWangFE();
    fv = lwfe->run(file, pcasys);
    dim = lwfe->getDimFV();
    if (fv[0] == -1)        
        cout << "Fingerprint Rejected! Wang" << endl;
    delete lwfe;
}

int LlamaMetodos::doRejectJain(string file){
    FingerCode *fc = new FingerCode();
    fv = fc->runReject(file, pcasys);
    dim = fc->getDimFV();

    delete fc;
}

int LlamaMetodos::doRejectJainPoincare(string file) {
    FingerCode *fc = new FingerCode();
    fv = fc->runReject(file, pcasys, true);
    dim = fc->getDimFV();

    delete fc;
}

int LlamaMetodos::doRejectHong(string file){
    HongFeatureExtractor *hfe = new HongFeatureExtractor();
    fv = hfe->runReject(file, pcasys);
    dim = hfe->getDimFV();

    delete hfe;
}

int LlamaMetodos::doLIUSegmentationPoincareWithJainReject(string file) {
    doRejectJainPoincare(file);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doLIUSegmentationPoincare(file);
    }      
    else {
        cout << "Fingerprint Rejected! Jain" << endl;
        fv[0] = -1;
    }
    

}


int LlamaMetodos::doZhangWithJainReject(string file) {
    doRejectJainPoincare(file);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doZhang(file);
    }      
    else {
        cout << "Fingerprint Rejected! Jain" << endl;
        fv[0] = -1;
    }
}

int LlamaMetodos::doCappelli2WithJainReject(string file) {    
    bool miAuxBool = ImgUtil::doAdjust;
    bool miAuxF = ImgUtil::fAdjust;
    ImgUtil::setDoAdjust(true);
    ImgUtil::setfAdjust(0.5625);
    doRejectJainPoincare(file);
    ImgUtil::setDoAdjust(miAuxBool);
    ImgUtil::setfAdjust(miAuxF);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doCappelli2(file);
    }      
    else {
        cout << "Fingerprint Rejected! Jain" << endl;
        fv[0] = -1;
    }
}

int LlamaMetodos::doRejectArea(string file) {
    NewPoincareSingularPoints *npsp = new NewPoincareSingularPoints();
    fv = MemUtil::createVector<float>(1);
    fv[0] = npsp->runReject(file, pcasys);
    dim = 1;
    delete npsp;

}

int LlamaMetodos::doLIUSegmentationPoincareWithAreaReject(string file) {
    doRejectArea(file);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doLIUSegmentationPoincare(file);
    }      
    else {
        cout << "Fingerprint Rejected! Area" << endl;
        fv[0] = -1;
    }
    

}


int LlamaMetodos::doZhangWithAreaReject(string file) {
    doRejectArea(file);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doZhang(file);
    }      
    else {
        cout << "Fingerprint Rejected! Area" << endl;
        fv[0] = -1;
    }
}

int LlamaMetodos::doJainPoincareWithAreaReject(string file) {
    doRejectArea(file);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doJAINPoincare(file);
    }      
    else {
        cout << "Fingerprint Rejected! Area" << endl;
        fv[0] = -1;
    }
}

int LlamaMetodos::doKaruWithAreaReject(string file) {
    doRejectArea(file);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doKARU(file);
    }      
    else {
        cout << "Fingerprint Rejected! Area" << endl;
        fv[0] = -1;
    }
}

int LlamaMetodos::doKaruWithJainReject(string file) {
    doRejectJainPoincare(file);
    if (fv[0] == 1) {// not rejected
        this->clear();
        doKARU(file);
    }      
    else {
        cout << "Fingerprint Rejected! Jain" << endl;
        fv[0] = -1;
    }
}

int LlamaMetodos::doGetEnhanced(string file) {
    GuardarRealce *gr = new GuardarRealce();
    gr->run(file, pcasys, 100, 100); // MO y VO = 100 son el ajuste de Zhang
    //gr->leer();
    fv = MemUtil::createVector<float>(1);
    fv[0]=-1;
    dim = 1;
    delete gr;
}