/* 
 * File:   MemUtil.hpp
 * Author: jcerron
 *
 * Created on 4 de abril de 2012, 19:04
 */

 /* 
 * \file        MemUtil.hpp
 * \author      Mikel Galar Idoate (mikel.galar@unavarra.es)
 * \author      Juan Cerrón González (juan.cerron@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * Implementation of util functions to manage memory
 *
 * 
 */

#ifndef MEMUTIL_HPP
#define	MEMUTIL_HPP

#include <iostream>
#include <cstring>
#include <cmath>

using namespace std;

namespace MemUtil{
    
    template<typename T> T *createVector(int columnas){
        if (columnas==0){
            cout << "error no se puede crear vector para dimension cero" << endl;
            return 0;
        }        
        T *matriz = new T [columnas];
        memset(matriz,0,columnas*sizeof(T));
        
        return matriz;
        
    }
    
    template<typename T> T *createVector(int columnas,T inic){
        if (columnas==0){
            cout << "error no se puede crear vector para dimension cero" << endl;
            return 0;
        }        
        T *matriz = new T [columnas];
        for (int i = 0; i < columnas; i++)
            matriz[i]=inic;
        
        return matriz;
        
    }
    
    template<typename T> T **create2DMatrix(int filas, int columnas){
        if (filas*columnas==0){
            cout << "error no se puede crear matriz para los datos " << endl;
            return 0;
        }
        T **matriz = new T *[filas];
        matriz[0] = new T [filas * columnas];
        
        for (int i = 1; i < filas; i++)            
            matriz[i] = matriz[i-1]+columnas;            
        
        memset(matriz[0],0,filas*columnas*sizeof(T));
        return matriz;
        
    }
    
    template<typename T> T **create2DMatrix(int filas, int columnas, T inic){
        if (filas*columnas==0){
            cout << "error no se puede crear matriz para los datos" << endl;
            return 0;
        }
        T **matriz = new T *[filas];
        matriz[0] = new T [filas * columnas];
        
        for (int i = 1; i < filas; i++){            
            matriz[i] = matriz[i-1]+columnas;
            
            for (int j = 0; j < columnas; j++)
                matriz[i][j] = inic;
        }
        
        for (int i = 0; i < columnas; i++)
            matriz[0][i] = inic;
        
        return matriz;
        
    }
    
    template<typename T> T **copy2DMatrix(T **originalMatrix, int filas, int columnas){
        if (filas*columnas==0){
            cout << "error no se puede crear matriz para los datos " << endl;
            return 0;
        }
        T **matriz = new T *[filas];
        matriz[0] = new T [filas * columnas];
        
        for (int i = 1; i < filas; i++)            
            matriz[i] = matriz[i-1]+columnas;            
        
        memcpy(matriz[0], originalMatrix[0], filas * columnas * sizeof(T));
        
        return matriz;
        
    }
    
    template<typename T> T ***create3DMatrix(int dimension, int filas, int columnas){
        if (filas*columnas*dimension==0){
            cout << "error no se puede crear tres d matriz para los datos " << endl;
            return 0;
        }

        T ***matriz = new T **[dimension];
              
        matriz[0] = new T *[filas * dimension];
        matriz[0][0] = new T [filas * columnas * dimension];
        
        for (int i = 0; i < dimension; i++) {
            if (i != 0) {
                matriz[i] = matriz[i - 1] + filas;
                matriz[i][0] = matriz[i - 1][0] + filas * columnas;
            }
            for (int j = 1; j < filas; j++) {
                matriz[i][j] = matriz[i][j - 1] + columnas;
            }
        }
        
        memset(matriz[0][0],0,filas*columnas*dimension*sizeof(T));
        return matriz;
        
    }
    
    template<typename T> T ***create3DMatrix(int dimension, int filas, int columnas, T inic){
        if (filas*columnas*dimension==0){
            cout << "error no se puede crear tres d matriz para los datos " << endl;
            return 0;
        }

        T ***matriz = new T **[dimension];
              
        matriz[0] = new T *[filas * dimension];
        matriz[0][0] = new T [filas * columnas * dimension];
        
        for (int i = 0; i < dimension; i++) {
            if (i != 0) {
                matriz[i] = matriz[i - 1] + filas;
                matriz[i][0] = matriz[i - 1][0] + filas * columnas;
            }
            for (int j = 1; j < filas; j++) {
                matriz[i][j] = matriz[i][j - 1] + columnas;
                
                for (int k = 0; k < columnas; k++)
                    matriz[i][j][k] = inic;
            }
            for (int k = 0; k < columnas; k++)
                matriz[i][0][k]=inic;
        }
        
        return matriz;
        
    }
    
    template<typename T> void showMatrix(T **matrix, int filas, int columnas){

        for (int i = 0; i < filas; i++){
            for (int j = 0; j < columnas; j++)
                cout << matrix[i][j] << " ";            
            cout << endl;
        }
        
    }
    
    template<typename T> void showMatrix(T *matrix, int columnas){
        
        for (int j = 0; j < columnas; j++)
            cout << matrix[j] << " ";        
        cout << endl;
    }
    
    template<typename T> void showMatrix(T ***matrix, int dimension, int filas, int columnas){
        
        for (int k = 0; k < dimension; k++){
            
            cout << "Dimension " << k+1 << endl;
            
            for (int i = 0; i < filas; i++){
                for (int j = 0; j < columnas; j++)
                    cout << matrix[k][i][j] << " ";
                cout << endl;
            }
            cout << endl;
        }     
    }
        
    template<typename T> int freeMatrix(T ****matrix){
        if (*matrix!=0){
            delete[] (*matrix)[0][0];
            delete[] (*matrix)[0];
            delete[] *matrix;
            *matrix = 0;
            return 0;
        }
        return 1;
    }
            
    template<typename T> int freeMatrix(T ***matrix){
        if (*matrix!=0){
            delete[] (*matrix)[0];
            delete[] *matrix;
            *matrix = 0;
            return 0;
        }
        return 1;
    }
    
    template<typename T> int freeMatrix(T **matrix){
        if (*matrix!=0){
            delete[] *matrix;
            *matrix = 0;
            return 0;
        }
        return 1;
    }

}

#endif	/* MEMUTIL_HPP */

