 /* 
 * \file        OrientationFFTPark.cpp
 * \author      Miguel Pagola Barrio (miguel.pagola@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * The implementation of OrientationFFTPark (Park) class
 *
 * 
 */

#include "fftw3.h"

#include "OrientationFFTPark.h"
#include "MemUtil.hpp"
#include "ImgUtil.h"
#include "FingerprintImg.h"

#include "ParametersPark.h"


OrientationFFTPark::OrientationFFTPark(){
    img = 0;
    imgOrig = 0;
    
    this->h = -1;
    this->w = -1;

    orH = 0;
    orW = 0;

    O = 0;
    D = 0;
    U = 0;

    xCos = 0;
    ySin = 0;
    
    corei = 0;
    corej = 0;
    
    plotMatrix = 0;
    dim = 0;
    F = 0;
}

OrientationFFTPark::OrientationFFTPark(float **image, int h, int w) {
    img = MemUtil::create2DMatrix<float>(h, w);
    for (int i = 0; i < h; i++)
        for (int j = 0; j < w; j++){
            img[i][j] = (float) image[i][j];
        }

    imgOrig = 0;
    
    this->h = h;
    this->w = w;

    orH = 0;
    orW = 0;

    O = 0;
    D = 0;
    U = 0;

    xCos = 0;
    ySin = 0;
    
    corei = 0;
    corej = 0;
    
    plotMatrix = 0;
    dim = 0;
    F = 0;
}

OrientationFFTPark::OrientationFFTPark(float **image, int h, int w, unsigned char **imageOrig){
    img = MemUtil::create2DMatrix<float>(h, w);
    imgOrig = MemUtil::create2DMatrix<float>(h, w);
    for (int i = 0; i < h; i++)
        for (int j = 0; j < w; j++){
            img[i][j] = (float) image[i][j];
            imgOrig[i][j] = (float) imageOrig[i][j];
        }

    this->h = h;
    this->w = w;

    orH = 0;
    orW = 0;

    O = 0;
    D = 0;
    U = 0;

    xCos = 0;
    ySin = 0;
    
    corei = 0;
    corej = 0;
    
    plotMatrix = 0;
    dim = 0;
    F = 0;
}


OrientationFFTPark::~OrientationFFTPark() {
 
    MemUtil::freeMatrix(img);    
    MemUtil::freeMatrix(O);    
    MemUtil::freeMatrix(xCos);    
    MemUtil::freeMatrix(ySin); 
    MemUtil::freeMatrix(D);
    MemUtil::freeMatrix(U);
    MemUtil::freeMatrix(F);
    MemUtil::freeMatrix(plotMatrix);
}

float* OrientationFFTPark::run(string file, PCASYSManager *pcasys, bool LargeOr){
    
    FingerprintImg fi;
    float *featurevector;
   
    if (ImgUtil::fAdjust != 1){
        
        fi.setPCASYSManager(pcasys);     
        fi.readFingerprintImage(file);  
        fi.setSegmentedPCASYS(fi.getImage(), fi.getH(), fi.getW());
    }
    else{

        fi.setPCASYSManager(pcasys);
        fi.readFingerprintImage(file);
        fi.segment();
    }
 
    h = fi.getH();
    w = fi.getW();
    
    unsigned char *image;
    img = MemUtil::create2DMatrix<float>(h, w);
    image = fi.getImage();
    int z = 0;
    for (int i = 0; i < h; i++)
        for (int j = 0; j < w; j++){
            img[i][j] = (float) image[z];
            z++;
        }
    
    // If the image size is 512x512 the method generate 31x31 blocks
    // Calculate the factor to adapt sizes;
    if (ImgUtil::DEBUG)
        cout << "h: " << h << " w: " << w << endl;
    float ffil = h/512.0;
    float fcol = w/512.0;
    
    // Therefore for block length size we takt the lowest value
    
    int blockheigth = (int)floor(32*ffil);
    int blockwidth = (int)floor(32*fcol);
    
    int bW = blockwidth;
    if (blockheigth > bW) bW = blockheigth;
    // bW = 8;
    if (ImgUtil::DEBUG)
        cout << "bW: " << bW << endl;
    
    // Size of Orientation image:
    // If the image is 512x512 there are 31x31 interior points, 
    // so Orientation image is 31x31
    int p = floor(bW/2);
    orH = h/p - 1;
    orW = w/p - 1;
    if (ImgUtil::DEBUG)
        cout << "orH : " << orH << " orW : " << orW  << endl;    
    
    // Get memory for matrix O and D
    O = MemUtil::create2DMatrix<float>(orH, orW);
    D = MemUtil::create2DMatrix<float>(orH, orW);
    // Create example Window
    float **W;
    W = MemUtil::create2DMatrix<float>(bW, bW);
    
    
    //int ii = 160;
    //int jj = 210;
   
    // Cargar los filtros
    F = createFilters(bW);
    if (ImgUtil::DEBUG)
        cout << "Filtros creados" << endl; 
    float *result;
    // Procces all the image and construct matrix O and D
    int x, y;
    x = 0;
    z = 0;
    for (int ii = p; ii < h; ii += p){
        y = 0;
        for (int jj = p; jj < w; jj+= p){
                for (int i = 0; i < bW; i++){
                    for (int j = 0; j < bW; j++){
                        W[i][j] = img[ii-p+i][jj-p+j];
                        // cout << W[i][j] << " ";
                    }
                    // cout << endl;
                }
                result = ComputeAngleBlock(W, bW);
                //cout << "Theta: " << result[0] << " D: " << result[1] << endl; 
                O[x][y] = result[0];
                D[x][y] = result[1];
                y++;
                z++;
        }
        x++;
    }
    if (ImgUtil::DEBUG)
        cout << "Matrices O y D creadas" << endl; 
    
    // Smooth
    xCos = MemUtil::create2DMatrix<float>(orH, orW);
    ySin = MemUtil::create2DMatrix<float>(orH, orW);
    
    for (int i = 0; i < orH; i++) {
        for (int j = 0; j < orW; j++) {
            //cout << " " << O[i][j];
        }
        //cout << endl;
    }
    for (int i = 0; i < orH; i++) {
        for (int j = 0; j < orW; j++) {
            xCos[i][j] = cos(2. * O[i][j]*PI/180.);
            ySin[i][j] = sin(2. * O[i][j]*PI/180.);
        }
    }
    if (ImgUtil::DEBUG) {
        string fileName2; 
        fileName2 = "OrientationSinCos";
        ImgUtil::writeOrImg(xCos, ySin, orW, orH, fileName2);
    }

    /* Smooth orientations */
    ImgUtil::smoothOrientations(O, &xCos, &ySin, orH, orW, WsmoothPark, false);
    if (ImgUtil::DEBUG)
        cout << "Smooth orientations done" << endl;    
   for (int i = 0; i < orH; i++) {
        for (int j = 0; j < orW; j++) {
            
            O[i][j] = 0.5 * atan2(ySin[i][j], xCos[i][j]);
            
            if (O[i][j] > PI / 2.)
                O[i][j] -= PI;
            if (O[i][j] < 0)
                O[i][j] = PI+O[i][j];
            O[i][j] = O[i][j]*180/PI;
            //cout << " " << (int)O[i][j];
        }
        //cout << endl;
    }
    
    if (ImgUtil::DEBUG) {        
        string fileName2 = "OrientationSinCosSmooth";
        ImgUtil::writeOrImg(xCos, ySin, orW, orH, fileName2);
    }
    
    
    computecore();
    if (ImgUtil::DEBUG)
        cout << "Calculado el core" << endl;    
    // extract 21x21 matrix  
    int sup, izd;
    sup = corei-10;
    izd = corej-10;
    plotMatrix = MemUtil::create2DMatrix<int>(21, 21);
    for (int i=0; i< 21; i++){
        for (int j=0; j < 21; j++){
            plotMatrix[i][j] = (int)O[sup+i][izd+j];
            //cout << plotMatrix[i][j] << " ";
        }
        //cout << endl;
    }
    // ImgUtil::writeGrayScaleImageCore(img, h, w, "core", corej*16, corei*16);
    
    if (LargeOr){
        featurevector = CreateLargeMatrix();
        return featurevector;
        
    }else{
        dim = 21*21;
        featurevector = MemUtil::createVector<float>(dim);
        int z = 0;
        for (int i=0; i< 21; i++){
            for (int j=0; j < 21; j++){
                featurevector[z] = plotMatrix[i][j];
                z++;
            }
         }
        return featurevector;
    }
 

}


float **OrientationFFTPark::computecore(){
    // The core is detected using the consistency


    U = MemUtil::create2DMatrix<float>(orH, orW);
    /*  
    for (int i = 0; i < orH; i++) {
        for (int j = 0; j < orW; j++) {            
            xCos[i][j] = cos(2. * O[i][j]*PI/180.);
            ySin[i][j] = sin(2. * O[i][j]*PI/180.);                   
        }
    }
    */ 
    float x, y;
    for (int i = 0; i < orH; i++) {
        for (int j = 0; j < orW; j++) { 
            x = 0;
            y = 0;
            for(int ii = -1; ii <= 1; ii++){
                for(int jj = -1; jj <= 1; jj++){
                    if ((i+ii > 0) && (j+jj > 0) && (i+ii < orH) && (j+jj < orW)){
                        x += xCos[i+ii][j+jj];
                        y += ySin[i+ii][j+jj];
                    }
                }
            }
            U[i][j] = sqrt(x*x + y*y);
            //cout << U[i][j] << " ";
        }
        //cout << endl;
    }
    
    // In order to find the core point is searched 
    // between fils and cols 11 and 21
    float ucore = 1000000;
    float usum;

    
    for (int i = 11; i < orH-11; i++) {
        for (int j = 11; j < orW-11; j++) { 
            usum = 0;
            for(int ii = -1; ii <= 0; ii++){
                for(int jj = -1; jj <= 0; jj++){
                    if ((i+ii > 0) && (j+jj > 0) && (i+ii < orH) && (j+jj < orW)){
                        usum += U[i+ii][j+jj];
                    }
                }
            }
            //cout << usum << " ";
            if (usum < ucore){
                ucore = usum;
                corei = i;
                corej = j;
            }
        }
        //cout << endl;
    }
    
    // cout << "Core: " << corei << " " << corej << endl;
   
}

float ***OrientationFFTPark::createFilters(int bW){
    
    int pos = floor(bW/2);
    
    int theta;

    float u, v, ep;
    float ***Filters;
    
    Filters = MemUtil::create3DMatrix<float>(18, bW, bW);
    // For every angle  theta a filter is constructed
    //theta = 50;
    for (theta = 0; theta <= 17; theta++){
        //cout << "Filtro theta: " << theta << endl;
        for (int i = -pos; i < pos; i++){
            for (int j = -pos; j < pos; j++){
                v = i*cos(10*theta*PI/180) + j*sin(10*theta*PI/180);
                u = sqrt((i*i) + (j*j) - (v*v));
                ep = -((u*u)/(FaPark*FaPark)) - ((v*v)/(FbPark*FbPark));
                if ((((u*u)/(FaPark*FaPark)) + ((v*v)/(FbPark*FbPark))) <= 1)
                    Filters[theta][i+pos][j+pos]=exp(ep);
                else
                    Filters[theta][i+pos][j+pos]=0;
                
                //cout << D[theta][i+pos][j+pos] << " ";
            }
            //cout << endl;
        }     
    }
    return Filters;
    
}

float ***OrientationFFTPark::directionBlocks(){

    float u, v;
    float ***M;
    int theta;
    
    M = MemUtil::create3DMatrix<float>(180, 5, 5);
    for (theta = 0; theta < 180; theta++){
        //cout << "Filtro: " << theta << endl;
        for (int i = -dBgPark; i <= dBgPark; i++){
            for (int j = -dBgPark; j <= dBgPark; j++){
                v = i*cos(theta*PI/180) + j*sin(theta*PI/180);
                u = sqrt(i*i+j*j-v*v);
                if ((u*u)/(dBaPark*dBaPark) <= 1){
                    M[theta][i+dBgPark][j+dBgPark] = dBcPark*exp(-(u*u)/(dBaPark*dBaPark));
                }else
                    M[theta][i+dBgPark][j+dBgPark] = 0;
                
                //cout << M[theta][i+g][j+g] << " ";
            }
            //cout << endl;
        }
    }
    return M;
}

float *OrientationFFTPark::CreateLargeMatrix(){
    float ***blocks;
    int sb;
    int **LargeMatrix;
    int theta;
    
    blocks = directionBlocks();
    
    sb = 5; // size block 5
    LargeMatrix = MemUtil::create2DMatrix<int>(21*sb, 21*sb);
    
    //cout << "Create Large Matrix" << endl;
    for(int i=0; i< 21; i++){
        for (int j = 0; j < 21; j++){
            theta = plotMatrix[i][j];
            //cout << "**theta: " << theta << endl;
            if (theta < 90)
                theta = theta + 90;
            else
                theta = theta - 90;
            for (int ii = 0; ii < sb; ii++){
                for (int jj = 0; jj < sb; jj++){
                    LargeMatrix[i*sb+ii][j*sb+jj] = (int)blocks[theta][ii][jj];//*25;
                }
            }
        }
    }
    
    for (int i = 0; i < 21*sb; i++){
        for (int j = 0; j < 21*sb; j++){
            cout << LargeMatrix[i][j] << " ";
        }
        //cout << endl;
    }
    
    
    float *featurevector;
    featurevector = MemUtil::createVector<float>(21*sb*21*sb);
    dim = 21*sb*21*sb;

    int z = 0;
    for (int i = 0; i < 21*sb; i++){
        for (int j = 0; j < 21*sb; j++){
            featurevector[z] = LargeMatrix[i][j];
            z++;
        }
        //cout << endl;
    }
    if (ImgUtil::DEBUG)
         ImgUtil::writeGrayScaleImage(LargeMatrix, 105, 105, "LargeMatrix", 1);
    MemUtil::freeMatrix(blocks);
    MemUtil::freeMatrix(LargeMatrix);
    return featurevector;
    
    
}
     

float *OrientationFFTPark::ComputeAngleBlock(float **W, int bW){
    
    int pos = floor(bW/2);
    int N = bW*bW;
    
    fftw_complex *in, *out;
    fftw_plan p;
    float **Mag, **MagS;
    int i, j, z;
    
    in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * N);
    out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * N);
    
    z=0;
    //cout << "Matriz de entrada: " << endl;
    for (int i = 0; i < bW; i++){
        for (int j = 0; j < bW; j++){
            in[z][0] = W[i][j];
            in[z][1] = 0;
            // cout << in[z][0] << " " << in[z][1] << " ";
            z++;
        }
        // cout << endl;
    }
    
    p = fftw_plan_dft_2d(bW, bW, in, out, FFTW_FORWARD, FFTW_ESTIMATE);
    fftw_execute(p);  
    z=0;
    // Calcular la magnitud de la FFT
    
    Mag = MemUtil::create2DMatrix<float>(bW+1, bW+1);
    MagS = MemUtil::create2DMatrix<float>(bW+1, bW+1);
    
    z = 0;
    for (i = 0; i < bW; i++){
        for (j = 0; j < bW; j++){
            Mag[i][j] = sqrt(out[z][0]*out[z][0] + out[z][1]*out[z][1]);
            z++;
        }
    }

        
   // shift matrix Mag
    int impar = 1;
    if (bW%2 == 0)
        impar = 0;
    
    for (int i = 0; i < pos ; i++){
        for (int j = 0; j < pos; j++){
            MagS[i][j] = Mag[i+pos+impar][j+pos+impar];
        }
    }
    
    for (int i = pos; i < bW ; i++){
        for (int j = 0; j < pos; j++){
            MagS[i][j] = Mag[i-pos][j+pos+impar];
        }
    }
    
    for (int i = 0; i < pos ; i++){
        for (int j = pos; j < bW; j++){
            MagS[i][j] = Mag[i+pos+impar][j-pos];
        }
    }
    
    for (int i = pos; i < bW ; i++){
        for (int j = pos; j < bW; j++){
            MagS[i][j] = Mag[i-pos][j-pos];
        }
    }
    
    int theta;
           
    float f[18];
    float fmax = 0;
    int thetamax;
    
    for (theta = 0; theta <= 17; theta++){
        f[theta] = 0;
        for(i = 0; i < bW; i++){
            for(j = 0; j < bW; j++){
                f[theta] = f[theta] + (MagS[i][j]*F[theta][i][j]);
                //cout << "Mag[i][j]: "<< MagS[i][j] << " F[theta][i][j]:" << F[theta][i][j] << "f[theta]: " << f[theta] << endl;
            }
        }
        // cout << "********f[theta]: " << f[theta] << endl;
        if (f[theta] > fmax){
            fmax = f[theta];
            thetamax = theta*10;
        }
    }
    
    // compute the magnitude of the angle
    float D;
    D = 0;
    for (theta = 0; theta <= 17; theta++){
        D = D + ((fmax-f[theta])/fmax);
    }

    // liberar Mag, in, out, D
    //Free memory of Filters D
    fftw_destroy_plan(p);
    fftw_free(in); fftw_free(out);
    
    
    MemUtil::freeMatrix(Mag);
    MemUtil::freeMatrix(MagS);
    
    float *res;
    res = MemUtil::createVector<float>(2);
    res[0] = thetamax;
    res[1] = D;
    
    return res;
  
    
}
  