/* 
 * \file        OrientationFFTPark.h
 * \author      Mikel Galar Idoate (mikel.galar@unavarra.es)
 * \author      Miguel Pagola (miguel.pagola@unavarra.es)
 * \version     1.0
 * 
 * \section     DESCRIPTION
 * 
 * Header file for OrientationFFTPark class
 *
 * 
 */


#ifndef ORIENTATIONFFTPARK_H
#define	ORIENTATIONFFTPARK_H

#include "PCASYSManager.h"


class OrientationFFTPark {
public:
    OrientationFFTPark();
    OrientationFFTPark(float **image, int h, int w);
    OrientationFFTPark(float **image, int h, int w, unsigned char **imageOrig);
    OrientationFFTPark(const OrientationFFTPark& orig);
    virtual ~OrientationFFTPark();
    
     /**
     * Executes the method Park from the fingerprint in file
     * \param file fingerprint
     * \return feature vector
     */
    float* run(string file, PCASYSManager *pcasys, bool LargeOr);
    
    float ***createFilters(int bW);
    
    float *ComputeAngleBlock(float **W, int bW);
    
    float **computecore();
    
    float ***directionBlocks();
    
    float *CreateLargeMatrix();
    
    int getDimFV() const;
    
    
private:
    float **img;                //!< Reference to the fingerprint image data
    float **imgOrig;            //!< Original image when using enhanced hong image
    int h;                      //!< Height of the image
    int w;                      //!< Width of the image
    
    int corei, corej;           //!< Position of the core in the orientation image
    
    float **O;                  //!< Orientation of each block (in radians)
    float **D;                  //!< Directionality of each block
    float **U;                  //!< Consistency of each block
    int **plotMatrix;         //!<  Final orientation matrix 
    int dim;                     //!< Dimension of the feature vector
    float **xCos;               //!< cos(2 O) value of the cosine of the orientation (x-direction)
    float **ySin;               //!< sin(2 O) value of the sine of the orientation (y-direction)
    
    int orH;                    //!< Height of the orientation map
    int orW;                    //!< Width of the orientation map
    int blockW;                 //!< Block size used to compute the orientation map
    
    float ***F;                 //!< Filters matrix
};

inline int OrientationFFTPark::getDimFV() const{ return dim;}
#endif	/* ORIENTATIONFFTPARK_H */

